@@ -31,16 +31,14 @@ sub rebless_class {
 				$_->can("match_class") ?
 					( $_->match_class => $_ )
 						: ();
-			}# map { print "rebless_class checking plugin $_\n"; $_ }
-				grep m{${\(__PACKAGE__)}::[^:]*$},
-				__PACKAGE__->plugins,
+			} __PACKAGE__->plugins,
 		};
 	}
 	$map->{ref $object};
 }
 
 sub action_class {
-	my $action = shift;
+	my $object = shift;
 	our $action_classes;
 	if ( !$action_classes ) {
 		$action_classes = {
@@ -48,18 +46,13 @@ sub action_class {
 				$_->can("action") ?
 					($_->action => $_)
 						: ();
-			}# map { print "action_class checking plugin $_\n"; $_ }
-				grep m{^${\(__PACKAGE__)}::[^:]*$},
-			__PACKAGE__->plugins,
+			} __PACKAGE__->plugins,
 		};
 	}
-	$action_classes->{ $action };
-}
-
-sub REBLESS {
-
+	$action_classes->{ $object->action };
 }
 
+sub REBLESS { }
 sub BUILD {
 	my $self = shift;
 	if ( my $epp = $self->message ) {
@@ -67,10 +60,9 @@ sub BUILD {
 		$class = rebless_class( $epp->message );
 		if ( !$class and $epp->message and
 			     $epp->message->can("action") ) {
-			$class = action_class($epp->message->action);
+			$class = action_class($epp->message);
 		}
 		if ( $class ) {
-			#FIXME: use ->meta->rebless_instance
 			bless $self, $class;
 			$self->REBLESS;
 		}
@@ -111,9 +103,16 @@ method process( SRS::EPP::Session $session ) {
 	$self->session($session);
 
 	# default handler is to return an unimplemented message
-	return $self->make_response(code => 2101);
+	return SRS::EPP::Response::Error->new(
+		id => 2101,
+		server_id => $self->server_id,
+		extra => "Sorry, command not yet implemented.",
+		);
 }
 
+method to_srs() {
+	die("$self: EPP command must implement to_srs");
+}
 method notify( SRS::EPP::SRSResponse @rs ) {
 	my $result;
 	if ( my $server_id = eval {
@@ -123,7 +122,12 @@ method notify( SRS::EPP::SRSResponse @rs ) {
 		$self->server_id($server_id);
 	}
 }
-
+method next_backend_message() {
+	die("$self: EPP command must implement next_backend_message (if it's going to un-set 'done')");
+}
+method response() {
+	die("$self: EPP command must implement response to reply");
+}
 sub make_response {
 	my $self = shift;
 	my $type = "SRS::EPP::Response";
@@ -211,7 +215,6 @@ L<SRS::EPP::Response>
 # Local Variables:
 # mode:cperl
 # indent-tabs-mode: t
-# tab-width: 8
 # cperl-continued-statement-offset: 8
 # cperl-brace-offset: 0
 # cperl-close-paren-offset: 0
